@@ -1,15 +1,15 @@
 #!/usr/local/bin/perl
 #
-# Copyright (C) 1993-2000 Ken'ichi Fukamachi
+# Copyright (C) 1993-2001 Ken'ichi Fukamachi
 #          All rights reserved. 
 #               1993-1996 fukachan@phys.titech.ac.jp
-#               1996-2000 fukachan@sapporo.iij.ad.jp
+#               1996-2001 fukachan@sapporo.iij.ad.jp
 # 
 # FML is free software; you can redistribute it and/or modify
 # it under the terms of GNU General Public License.
 # See the file COPYING for more details.
 #
-# $Id: fml.pl,v 2.124 2001/01/09 12:40:07 fukachan Exp $
+# $FML: fml.pl,v 2.124.2.6 2001/10/15 12:53:04 fukachan Exp $
 
 
 ##### SubRoutines #####
@@ -273,6 +273,9 @@ sub LoadConfig
 
     require 'libsmtp.pl';		# a library using smtp
 
+    # load MIME handling functions for convenience
+    if ($LANGUAGE eq 'Japanese') { require 'libMIME.pl';}
+
     # if mode:some is set, load the default configuration of the mode
     for (keys %Envelope) { 
 	/^mode:(\S+)/ && $Envelope{$_} && do { &DEFINE_MODE($1);};
@@ -343,7 +346,7 @@ sub SetDefaults
 	 'References', 'Keywords', 'Comments', 'Encrypted',
 	 ':XMLNAME:', ':XMLCOUNT:', 'X-MLServer', 
 	 'XRef', 'X-Stardate', 'X-ML-Info', 
-	 'X-Mailer',
+	 'X-Mailer', 'X-Dispatcher', 'X-Newsreader', 'User-Agent',
 	 'Mail-Followup-To',	# I-D now?
 	 ':body:', ':any:', 
 	 'X-Authentication-Warning',
@@ -360,12 +363,13 @@ sub GetTime
     @WDay = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
     @Month = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 
 	      'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
-    ($sec,$min,$hour,$mday,$mon,$year,$wday) = (localtime(time))[0..6];
+    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
     $Now = sprintf("%02d/%02d/%02d %02d:%02d:%02d", 
 		   ($year % 100), $mon + 1, $mday, $hour, $min, $sec);
     $MailDate = sprintf("%s, %d %s %d %02d:%02d:%02d %s", 
 			$WDay[$wday], $mday, $Month[$mon], 
-			1900 + $year, $hour, $min, $sec, $TZone);
+			1900 + $year, $hour, $min, $sec, 
+			$isdst ? $TZONE_DST : $TZone);
 
     # /usr/src/sendmail/src/envelop.c
     #     (void) sprintf(tbuf, "%04d%02d%02d%02d%02d", tm->tm_year + 1900,
@@ -393,6 +397,8 @@ sub InitConfig
     # fml 4.0 modules
     push(@INC, "$ExecDir/module");
     push(@INC, "$ExecDir/module/CPAN");
+    push(@INC, "$ExecDir/module/Japanese");
+    push(@INC, "$ExecDir/module/fml-devel");
 
     # a little configuration before the action
     if (defined $FML_UMASK) {
@@ -459,9 +465,17 @@ sub InitConfig
 	&eval("-d \$$_||&Mkdir(\$$_);");
     }
 
-    for ($LOGFILE, $MEMBER_LIST, $MGET_LOGFILE, 
-	 $SEQUENCE_FILE, $SUMMARY_FILE, $LOG_MESSAGE_ID) {
-	-f $_ || &Touch($_);	
+    if ($USE_DATABASE) {
+	for ($LOGFILE, $MGET_LOGFILE, 
+	     $SEQUENCE_FILE, $SUMMARY_FILE, $LOG_MESSAGE_ID) {
+	    -f $_ || &Touch($_);
+	}	
+    }
+    else {
+	for ($LOGFILE, $MEMBER_LIST, $MGET_LOGFILE, 
+	     $SEQUENCE_FILE, $SUMMARY_FILE, $LOG_MESSAGE_ID) {
+	    -f $_ || &Touch($_);
+	}	
     }
 
     ### CFVersion 3
@@ -543,7 +557,7 @@ sub InitConfig
 sub Parsing { &Parse;}
 sub Parse
 {
-    $0 = "$FML: Parsing header and body <$LOCKFILE>";
+    $0 = "${FML}: Parsing header and body <$LOCKFILE>";
     local($bufsiz, $buf, $p, $maxbufsiz, $in_header);
 
     $maxbufsiz = &ATOI($INCOMING_MAIL_SIZE_LIMIT) if $INCOMING_MAIL_SIZE_LIMIT;
@@ -590,7 +604,7 @@ sub GetFieldsFromHeader
     local($field, $value, @hdr, %hf);
     local($s);
 
-    $0 = "$FML: GetFieldsFromHeader <$LOCKFILE>";
+    $0 = "${FML}: GetFieldsFromHeader <$LOCKFILE>";
 
     # To ensure non exsistence
     for (split(/\|/, $SKIP_FIELDS)) { &DELETE_FIELD($_);}
@@ -1168,7 +1182,7 @@ sub RuleSetTo
     local($ok, $match, $addr, $x_addr, $ml, $buf, $to);
     local(@ml) = ($MAIL_LIST, @MAIL_LIST_ALIASES); # PLAY_TO Trick;
 
-    eval "use Mail::Address;";
+    eval "require Mail::Address; Mail::Address->import();";
 
     # if fails, call fml 3.0 compatible routine.
     if ($@ ne '') {
@@ -1217,7 +1231,7 @@ sub Conv2mailbox
     local($mb, *e) = @_;	# original string
     local($addr);
 
-    eval "use Mail::Address;";
+    eval "require Mail::Address; Mail::Address->import();";
 
     # if fails, call fml 3.0 compatible routine.
     if ($@ ne '') {
@@ -1388,7 +1402,7 @@ sub Distribute
 
 sub RunStartHooks
 {
-    $0 = "$FML: RunStartHooks <$LOCKFILE>";
+    $0 = "${FML}: RunStartHooks <$LOCKFILE>";
 
     # additional before action
     $START_HOOK && &eval($START_HOOK, 'Start hook');
@@ -1396,14 +1410,14 @@ sub RunStartHooks
     for (keys %FmlStartHook) {
 	print STDERR "Run StartHook $_ -> $FmlStartHook{$_}\n" if $debug;
 	next unless $FmlStartHook{$_};
-	$0 = "$FML: Run FmlStartHook [$_] <$LOCKFILE>";
+	$0 = "${FML}: Run FmlStartHook [$_] <$LOCKFILE>";
 	&eval($FmlStartHook{$_}, "Run FmlStartHook [$_]");
     }
 }
 
 sub RunEndHooks
 {
-    $0 = "$FML: RunEndHooks <$LOCKFILE>";
+    $0 = "${FML}: RunEndHooks <$LOCKFILE>";
 
     # additional before action
     $END_HOOK && &eval($END_HOOK, 'END_HOOK');
@@ -1411,7 +1425,7 @@ sub RunEndHooks
     for (keys %FmlEndHook) {
 	print STDERR "Run EndHook $_ -> $FmlEndHook{$_}\n" if $debug;
 	next unless $FmlEndHook{$_};
-	$0 = "$FML: Run FmlEndHook [$_] <$LOCKFILE>";
+	$0 = "${FML}: Run FmlEndHook [$_] <$LOCKFILE>";
 	&eval($FmlEndHook{$_}, "Run FmlEndHook [$_]");
     }
 }
@@ -1421,28 +1435,28 @@ sub ExExec { &RunHooks(@_);}
 sub RunExitHooks
 {
     local($s);
-    $0 = "$FML: RunExitHooks <$LOCKFILE>";
+    $0 = "${FML}: RunExitHooks <$LOCKFILE>";
 
     # FIX COMPATIBILITY
     $FML_EXIT_HOOK .= $_PCB{'hook', 'str'};
 
     if ($s = $FML_EXIT_HOOK) {
 	print STDERR "\nmain::eval >$s<\n\n" if $debug;
-	$0 = "$FML: Run Hooks(eval) <$LOCKFILE>";
+	$0 = "${FML}: Run Hooks(eval) <$LOCKFILE>";
 	&eval($s, 'Run Hooks:');
     }
     
     for (keys %FmlExitHook) {
 	print STDERR "Run hooks $_ -> $FmlExitHook{$_}\n" if $debug;
 	next unless $FmlExitHook{$_};
-	$0 = "$FML: Run FmlExitHook [$_] <$LOCKFILE>";
+	$0 = "${FML}: Run FmlExitHook [$_] <$LOCKFILE>";
 	&eval($FmlExitHook{$_}, "Run FmlExitHook [$_]");
     }
 }
 
 sub ExecNewProcess
 {
-    $0 = "$FML: Run New Process <$LOCKFILE>";
+    $0 = "${FML}: Run New Process <$LOCKFILE>";
     $FML_EXIT_PROG .= $_PCB{'hook', 'prog'};
     if ($FML_EXIT_PROG) { &use('kernsubr2'); &__ExecNewProcess;}
 }
@@ -1513,7 +1527,26 @@ sub DoMailListMemberP
 	my (%mib, %result, %misc, $error);
 	&DataBaseMIBPrepare(\%mib, $action, {'address' => $addr});
 	&DataBaseCtl(\%Envelope, \%mib, \%result, \%misc);
-	if ($mib->{'error'}) { return 0;}
+
+       # if search fails for $MEMBER_LIST,
+       if ( !$mib{'_result'} && ($type eq 'm')) {
+           # retry search in $ADMIN_MEMBER_LIST
+           my (%xmib, %result, %misc, $error);
+           &DataBaseMIBPrepare(\%xmib,'admin_member_p',{'address' => $addr});
+           &DataBaseCtl(\%Envelope, \%xmib, \%result, \%misc);
+
+           # search fails for both $MEMBER_LIST and $ADMIN_MEMBER_LIST
+           if ($xmib{'error'}) {
+               return 0;
+           }
+           else {
+               $mib{ _result } = $xmib{ _result };
+           }
+       }
+       # if search fails for $ACTIVE_LIST, return here ASAP.
+       elsif ($mib{'error'}) {
+           return 0;
+       }
 
 	$Envelope{'database:cache:$action'} = 1 if $mib{'_result'};
 
@@ -1577,7 +1610,7 @@ sub MailListAdminMemberP
 	my (%mib, %result, %misc, $error);
 	&DataBaseMIBPrepare(\%mib, 'admin_member_p', {'address' => $addr});
 	&DataBaseCtl(\%Envelope, \%mib, \%result, \%misc); 
-	if ($mib->{'error'}) { return 0;}
+       if ($mib{'error'}) { return 0;}
 
 	$Envelope{'database:cache:admin_member_p'} = 1 if $mib{'_result'};
 
@@ -1825,6 +1858,20 @@ sub AddressMatch
     ($i >= $m) ? 1 : 0;
 }
 
+sub LowerDomain
+{
+    my ($addr) = @_;
+    my (@addr) = split(/\@/, $addr);
+
+    if ($DATABASE_DRIVER_ATTRIBUTES =~ /always_lower_domain/) {
+	$addr[1] =~ tr/A-Z/a-z/;
+	return $addr[0].'@'.$addr[1];
+    }
+    else {
+	return $addr;
+    }
+}
+
 ####### Section: Info
 # Recreation of the whole mail for error infomation
 sub WholeMail { 
@@ -2477,7 +2524,7 @@ sub PerlModuleExistP
     local($pm) = @_;
     if ($] !~ /^5\./) { &Log("ERROR: using $pm requires perl 5"); return 0;}
     eval("use $pm");
-    if ($@) { &Log("${pm}5.pm NOT FOUND; Please install ${pm}.pm"); return 0;}
+    if ($@) { &Log("${pm}.pm NOT FOUND; Please install ${pm}.pm"); return 0;}
     1;
 }
 
@@ -2606,9 +2653,9 @@ sub DBCtl
 sub MailLoopP
 {
     if ($Envelope{'h:x-ml-info:'}) {
-	if ($Envelope{'h:x-ml-info:'} =~ /$MAINTAINER/i ||
-	    $Envelope{'h:x-ml-info:'} =~ /$MAIL_LIST/i ||
-	    $Envelope{'h:x-ml-info:'} =~ /$CONTROL_ADDRESS/i) {
+	if ($Envelope{'h:x-ml-info:'} =~ /contact $MAINTAINER/i ||
+	    $Envelope{'h:x-ml-info:'} =~ /(address\s+|mailto:)$MAIL_LIST/i ||
+	    $Envelope{'h:x-ml-info:'} =~ /(address\s+|mailto:)$CONTROL_ADDRESS/i) {
 	    &Log("Loop Alert: dup X-ML-Info:");
 	    &WarnE("Loop Alert: dup X-ML-Info: $ML_FN", 
 		   "fml <$MAIL_LIST> has detected a loop condition so that\n"
@@ -2943,7 +2990,7 @@ sub Flock
     $FlockFile = $FlockFile ||
 	(open(LOCK,$FP_SPOOL_DIR) ? $FP_SPOOL_DIR : "$DIR/config.ph");
 
-    $0 = "$FML: Locked(flock) until $ut <$LOCKFILE>";
+    $0 = "${FML}: Locked(flock) until $ut <$LOCKFILE>";
 
     # spool is also a file!
     if (! open(LOCK, $FlockFile)) {
@@ -2955,7 +3002,7 @@ sub Flock
 
 sub Funlock 
 {
-    $0 = "$FML: Unlock <$LOCKFILE>";
+    $0 = "${FML}: Unlock <$LOCKFILE>";
 
     flock(LOCK, $LOCK_UN);
     close(LOCK); # unlock,close <kizu@ics.es.osaka-u.ac.jp>
@@ -2965,7 +3012,7 @@ sub Funlock
 sub TimeOut
 {
     &GetTime;
-    $0 = "$FML: TimeOut $Now <$LOCKFILE>";
+    $0 = "${FML}: TimeOut $Now <$LOCKFILE>";
 
     # Now we may be not able to connect socket, isn't it?
     # &WarnE("TimeOut: $MailDate ($From_address) $ML_FN", $NULL);
@@ -3053,7 +3100,7 @@ sub Tick
 {
     local($cur, $fp, $qp);
 
-    &GetTime; $0 = "$FML: Tick $Now <$LOCKFILE>";
+    &GetTime; $0 = "${FML}: Tick $Now <$LOCKFILE>";
 
     return unless $HAS_ALARM;
 
